"""A Web browser that can be driven by an application."""

__metaclass__ = type
__all__ = [
    'Browser',
    'Command',
    ]
import os

# Suppress accessibility warning because the test runner does not have UI.
os.environ['GTK_MODULES'] = ''

try:
    from gi.repository import GObject
    from gi.repository import Gtk
    from gi.repository import WebKit
    # Hush lint
    GObject, Gtk, WebKit
except ImportError:
    # Support for lucid.
    import pygtk
    pygtk.require("2.0")
    import glib as GObject
    import gtk as Gtk
    import webkit as WebKit
    # XXX sinzui 2011-06-16 LP:27112:
    # This evil encoding fix undoes the evil done by import gtk.
    import sys
    reload(sys)
    sys.setdefaultencoding('ascii')


class Command:
    """A representation of the status and result of a command."""
    STATUS_RUNNING = object()
    STATUS_COMPLETE = object()

    CODE_UNKNOWN = -1
    CODE_SUCCESS = 0
    CODE_FAIL = 1

    def __init__(self, status=STATUS_RUNNING, return_code=CODE_UNKNOWN,
                 content=None):
        self.status = status
        self.return_code = return_code
        self.content = content


class Browser(WebKit.WebView):
    """A browser that can be driven by an application."""

    STATUS_PREFIX = '::::'
    TIMEOUT = 5000

    def __init__(self, show_window=False, hide_console_messages=True):
        super(Browser, self).__init__()
        self.show_window = show_window
        self.hide_console_messages = hide_console_messages
        self.browser_window = None
        self.script = None
        self.command = None
        self.listeners = {}
        self._connect('console-message', self._on_console_message, False)

    def load_page(self, uri, timeout=TIMEOUT):
        """Load a page and return the content."""
        self._setup_listening_operation(timeout)
        if uri.startswith('/'):
            uri = 'file://' + uri
        self.load_uri(uri)
        Gtk.main()
        return self.command

    def run_script(self, script, timeout=TIMEOUT):
        """Run a script and return the result."""
        self._setup_listening_operation(timeout)
        self.script = script
        self._connect('notify::load-status', self._on_script_load_finished)
        self.load_string(
            '<html><head></head><body></body></html>',
            'text/html', 'UTF-8', 'file:///')
        Gtk.main()
        return self.command

    @staticmethod
    def escape_script(text):
        """Escape the text so that it can be interpolated in to JS."""
        return text.replace(
            '\\', '\\\\').replace('"', '\\"').replace("'", "\\'").replace(
            '\n', '\\n')

    def _setup_listening_operation(self, timeout):
        """Setup a one-time listening operation for command's completion."""
        self._create_window()
        self.command = Command()
        self._connect(
            'status-bar-text-changed', self._on_status_bar_text_changed)
        GObject.timeout_add(timeout, self._on_timeout)

    def _create_window(self):
        """Create a window needed to render pages."""
        if self.browser_window is not None:
            return
        self.browser_window = Gtk.Window()
        self.browser_window.set_default_size(800, 600)
        self.browser_window.connect("destroy", self._on_quit)
        scrolled = Gtk.ScrolledWindow()
        scrolled.add(self)
        self.browser_window.add(scrolled)
        if self.show_window:
            self.browser_window.show_all()

    def _on_quit(self, widget=None):
        Gtk.main_quit()

    def _on_status_bar_text_changed(self, view, text):
        if text.startswith(self.STATUS_PREFIX):
            self._disconnect('status-bar-text-changed')
            self.execute_script('window.status = "";')
            self.command.status = Command.STATUS_COMPLETE
            self.command.return_code = Command.CODE_SUCCESS
            self.command.content = text[4:]
            self._on_quit()

    def _on_script_load_finished(self, view, load_status):
        # pywebkit does not have WebKit.LoadStatus.FINISHED.
        statuses = ('WEBKIT_LOAD_FINISHED', 'WEBKIT_LOAD_FAILED')
        if self.props.load_status.value_name not in statuses:
            return
        self._disconnect('notify::load-status')
        self.execute_script(self.script)
        self.script = None

    def _on_timeout(self):
        if self.command.status is not Command.STATUS_COMPLETE:
            self._disconnect()
            self.command.status = Command.STATUS_COMPLETE
            self.command.return_code = Command.CODE_FAIL
            self._on_quit()
        return False

    def _on_console_message(self, view, message, line_no, source_id, data):
        return self.hide_console_messages

    def _connect(self, signal, callback, *args):
        self.listeners[signal] = self.connect(signal, callback, *args)

    def _disconnect(self, signal=None):
        if signal is None:
            signals = self.listeners.keys()
        elif isinstance(signal, basestring):
            signals = [signal]
        for key in signals:
            self.disconnect(self.listeners[key])
            del self.listeners[key]
